from ._base import BaseImage


class Kafka(BaseImage):
    label = 'kafka'
    name = 'kafka'
    port = 9092

    def get_image_options(self):
        image_options = super().get_image_options()
        image_options.update(dict(
            environment={
                'ADVERTISED_PORT': '9092',
                'ADVERTISED_HOST': '0.0.0.0'
            },
            ports={
                f'9092': '9092',
                f'2181': '2181'
            }
        ))
        return image_options

    def check(self):
        try:
            from kafka import KafkaClient
            from kafka.common import KafkaUnavailableError
            KafkaClient(f"{self.host}:{self.get_port()}")
            return True
        except KafkaUnavailableError:
            pass
        return False


kafka_image = Kafka()



from ._base import BaseImage

import os
import requests


class ElasticSearch(BaseImage):
    name = 'elasticsearch'
    port = 9200

    def get_image_options(self):
        image_options = super().get_image_options()
        env = {
            'cluster.name': 'docker-cluster',
            'ES_JAVA_OPTS': '-Xms512m -Xmx512m',
            "xpack.security.enabled": "false",
            #"http.host=0.0.0.0" -e "transport.host=127.0.0.1"
        }
        if 'oss' in self.image:
            del env['xpack.security.enabled']
        image_options.update(dict(
            cap_add=['IPC_LOCK'],
            mem_limit='1g',
            environment=env
        ))
        if 'TRAVIS' in os.environ:
            image_options.update({
                'publish_all_ports': False,
                'ports': {
                    f'9200/tcp': '9200'
                }
            })
        return image_options

    def check(self):
        url = f'http://{self.host}:{self.get_port()}/'
        try:
            resp = requests.get(url)
            if resp.status_code == 200:
                return True
        except:
            pass
        return False


es_image = ElasticSearch()



from ._base import BaseImage


class Postgresql(BaseImage):
    name = 'postgresql'
    port = 5432

    def get_image_options(self):
        image_options = super().get_image_options()
        image_options.update(dict(
            environment={
                'POSTGRES_PASSWORD': '',
                'POSTGRES_DB': 'guillotina',
                'POSTGRES_USER': 'postgres'
            }
        ))
        return image_options

    def check(self):
        try:
            import psycopg2
            conn = psycopg2.connect(
                f"dbname=guillotina user=postgres host={self.host} "
                f"port={self.get_port()}")
            cur = conn.cursor()
            cur.execute("SELECT 1;")
            cur.fetchone()
            cur.close()
            conn.close()
            return True
        except: # noqa
            conn = None
            cur = None
        return False


pg_image = Postgresql()



from ._base import BaseImage
from time import sleep
import requests


class Minio(BaseImage):
    name = 'minio'
    port = 9000

    @property
    def image(self):
        return 'minio/minio'

    def get_image_options(self):
        image_options = super().get_image_options()

        image_options.update(dict(
            cap_add=['IPC_LOCK'],
            mem_limit='200m',
            command='server /export',
            environment={
                'MINIO_ACCESS_KEY': 'x' * 10,
                'MINIO_SECRET_KEY': 'x' * 10,
            },
            publish_all_ports=False,
            ports={
                '9000/tcp': '19000'
            }
        ))
        return image_options

    def check(self):
        url = f'http://{self.host}:{self.get_port()}/'
        try:
            resp = requests.options(url)
            if resp.status_code == 200:
                return True
        except:
            pass
        return False


minio_image = Minio()



from .containers.cockroach import cockroach_image
from .containers.es import es_image
from .containers.etcd import etcd_image
from .containers.pg import pg_image
from .containers.redis import redis_image
from .containers.rabbitmq import rabbitmq_image
from .containers.kafka import kafka_image
from .containers.minio import minio_image

import os
import pytest


IS_TRAVIS = 'TRAVIS' in os.environ


@pytest.fixture(scope='session')
def redis():
    """
    detect travis, use travis's postgres; otherwise, use docker
    """
    if IS_TRAVIS:
        host = 'localhost'
        port = 6379
    else:
        host, port = redis_image.run()

    yield host, port  # provide the fixture value

    if not IS_TRAVIS:
        redis_image.stop()


@pytest.fixture(scope='session')
def cockroach():
    yield cockroach_image.run()
    cockroach_image.stop()


@pytest.fixture(scope='session')
def pg():
    if IS_TRAVIS:
        host = 'localhost'
        port = 6379
    else:
        host, port = pg_image.run()

    yield host, port  # provide the fixture value

    if not IS_TRAVIS:
        pg_image.stop()


@pytest.fixture(scope='session')
def etcd():
    yield etcd_image.run()
    etcd_image.stop()


@pytest.fixture(scope='session')
def es():
    yield es_image.run()
    es_image.stop()


@pytest.fixture(scope='session')
def rabbitmq():
    yield rabbitmq_image.run()
    rabbitmq_image.stop()


@pytest.fixture(scope='session')
def kafka():
    yield kafka_image.run()
    kafka_image.stop()


@pytest.fixture(scope='session')
def minio():
    if IS_TRAVIS:
        host = 'localhost'
        port = 19000
    else:
        host, port = minio_image.run()

    yield host, port

    if not IS_TRAVIS:
        minio_image.stop()
